@@ -13,15 +13,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __POPPLER_DOCUMENT_H__
 #define __POPPLER_DOCUMENT_H__
 
 #include <glib-object.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-
 #include "poppler.h"
 
 G_BEGIN_DECLS
@@ -86,7 +84,8 @@ typedef enum /*< flags >*/
   POPPLER_PERMISSIONS_OK_TO_MODIFY = 1 << 1,
   POPPLER_PERMISSIONS_OK_TO_COPY = 1 << 2,
   POPPLER_PERMISSIONS_OK_TO_ADD_NOTES = 1 << 3,
-  POPPLER_PERMISSIONS_FULL = (POPPLER_PERMISSIONS_OK_TO_PRINT | POPPLER_PERMISSIONS_OK_TO_MODIFY | POPPLER_PERMISSIONS_OK_TO_COPY | POPPLER_PERMISSIONS_OK_TO_ADD_NOTES)
+  POPPLER_PERMISSIONS_OK_TO_FILL_FORM = 1 << 4,
+  POPPLER_PERMISSIONS_FULL = (POPPLER_PERMISSIONS_OK_TO_PRINT | POPPLER_PERMISSIONS_OK_TO_MODIFY | POPPLER_PERMISSIONS_OK_TO_COPY | POPPLER_PERMISSIONS_OK_TO_ADD_NOTES | POPPLER_PERMISSIONS_OK_TO_FILL_FORM)
 
 } PopplerPermissions;
 
@@ -103,6 +102,9 @@ PopplerDocument *poppler_document_new_from_data     (char             *data,
 gboolean         poppler_document_save              (PopplerDocument  *document,
 						     const char       *uri,
 						     GError          **error);
+gboolean         poppler_document_save_a_copy       (PopplerDocument  *document,
+						     const char       *uri,
+						     GError          **error);
 int              poppler_document_get_n_pages       (PopplerDocument  *document);
 PopplerPage     *poppler_document_get_page          (PopplerDocument  *document,
 						     int               index);
