@@ -6,6 +6,24 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2005, 2007, 2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005 Jonathan Blandford <jrb@redhat.com>
+// Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
+// Copyright (C) 2006 Kristian Høgsberg <krh@redhat.com>
+// Copyright (C) 2007 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef GLOBALPARAMS_H
 #define GLOBALPARAMS_H
 
@@ -13,9 +31,12 @@
 #pragma interface
 #endif
 
+#include <assert.h>
 #include "poppler-config.h"
 #include <stdio.h>
+#ifndef _MSC_VER
 #include <fontconfig/fontconfig.h>
+#endif
 #include "goo/gtypes.h"
 #include "CharTypes.h"
 
@@ -74,6 +95,14 @@ public:
   };
 
   DisplayFontParam(GooString *nameA, DisplayFontParamKind kindA);
+  void setFileName(GooString *fileNameA) {
+    if (displayFontT1 == kind)
+        t1.fileName = fileNameA;
+    else {
+        assert(displayFontTT == kind);
+        tt.fileName = fileNameA;
+    }
+  }  
   virtual ~DisplayFontParam();
 };
 
@@ -136,6 +165,10 @@ public:
 
   void setBaseDir(char *dir);
 
+#ifdef _MSC_VER
+  void setupBaseFonts(char *dir);
+#endif
+
   //----- accessors
 
   CharCode getMacRomanCharCode(char *charName);
@@ -190,6 +223,8 @@ public:
   GBool loadPlugin(char *type, char *name);
 #endif
 
+  GooList *getEncodingNames();
+
   //----- functions to set parameters
   void setPSExpandSmaller(GBool expand);
   void setPSShrinkLarger(GBool shrink);
@@ -209,6 +244,13 @@ public:
   GBool setEnableFreeType(char *s);
   GBool setAntialias(char *s);
   GBool setVectorAntialias(char *s);
+  void setStrokeAdjust(GBool strokeAdjust);
+  void setScreenType(ScreenType st);
+  void setScreenSize(int size);
+  void setScreenDotRadius(int radius);
+  void setScreenGamma(double gamma);
+  void setScreenBlackThreshold(double blackThreshold);
+  void setScreenWhiteThreshold(double whiteThreshold);
   void setMapNumericCharNames(GBool map);
   void setMapUnknownCharNames(GBool map);
   void setPrintCommands(GBool printCommandsA);
@@ -302,7 +344,9 @@ private:
   UnicodeMapCache *unicodeMapCache;
   CMapCache *cMapCache;
   
+#ifndef _MSC_VER
   FcConfig *FCcfg;
+#endif
 
 #ifdef ENABLE_PLUGINS
   GList *plugins;		// list of plugins [Plugin]
