@@ -4,6 +4,21 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2006 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007 Ilmari Heikkinen <ilmari.heikkinen@gmail.com>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include <config.h>
 
 #ifdef USE_GCC_PRAGMAS
@@ -20,7 +35,8 @@
 //------------------------------------------------------------------------
 
 SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPad,
-			   SplashColorMode modeA, GBool topDown) {
+			   SplashColorMode modeA, GBool alphaA,
+			   GBool topDown) {
   width = widthA;
   height = heightA;
   mode = modeA;
@@ -31,24 +47,16 @@ SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPad,
   case splashModeMono8:
     rowSize = width;
     break;
-  case splashModeAMono8:
-    rowSize = width * 2;
-    break;
   case splashModeRGB8:
   case splashModeBGR8:
     rowSize = width * 3;
     break;
-  case splashModeRGB8Qt:
-  case splashModeARGB8:
-  case splashModeBGRA8:
-#if SPLASH_CMYK
-  case splashModeCMYK8:
-#endif
+  case splashModeXBGR8:
     rowSize = width * 4;
     break;
 #if SPLASH_CMYK
-  case splashModeACMYK8:
-    rowSize = width * 5;
+  case splashModeCMYK8:
+    rowSize = width * 4;
     break;
 #endif
   }
@@ -59,6 +67,11 @@ SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPad,
     data += (height - 1) * rowSize;
     rowSize = -rowSize;
   }
+  if (alphaA) {
+    alpha = (Guchar *)gmalloc(width * height);
+  } else {
+    alpha = NULL;
+  }
 }
 
 
@@ -68,17 +81,29 @@ SplashBitmap::~SplashBitmap() {
   } else {
     gfree(data);
   }
+  gfree(alpha);
 }
 
+
 SplashError SplashBitmap::writePNMFile(char *fileName) {
   FILE *f;
-  SplashColorPtr row, p;
-  int x, y;
+  SplashError e;
 
   if (!(f = fopen(fileName, "wb"))) {
     return splashErrOpenFile;
   }
 
+  e = this->writePNMFile(f);
+  
+  fclose(f);
+  return e;
+}
+
+
+SplashError SplashBitmap::writePNMFile(FILE *f) {
+  SplashColorPtr row, p;
+  int x, y;
+
   switch (mode) {
 
   case splashModeMono1:
@@ -107,19 +132,6 @@ SplashError SplashBitmap::writePNMFile(char *fileName) {
     }
     break;
 
-  case splashModeAMono8:
-    fprintf(f, "P5\n%d %d\n255\n", width, height);
-    row = data;
-    for (y = 0; y < height; ++y) {
-      p = row;
-      for (x = 0; x < width; ++x) {
-	fputc(splashAMono8M(p), f);
-	p += 2;
-      }
-      row += rowSize;
-    }
-    break;
-
   case splashModeRGB8:
     fprintf(f, "P6\n%d %d\n255\n", width, height);
     row = data;
@@ -135,6 +147,22 @@ SplashError SplashBitmap::writePNMFile(char *fileName) {
     }
     break;
 
+  case splashModeXBGR8:
+    fprintf(f, "P6\n%d %d\n255\n", width, height);
+    row = data;
+    for (y = 0; y < height; ++y) {
+      p = row;
+      for (x = 0; x < width; ++x) {
+	fputc(splashBGR8R(p), f);
+	fputc(splashBGR8G(p), f);
+	fputc(splashBGR8B(p), f);
+	p += 4;
+      }
+      row += rowSize;
+    }
+    break;
+
+
   case splashModeBGR8:
     fprintf(f, "P6\n%d %d\n255\n", width, height);
     row = data;
@@ -150,63 +178,16 @@ SplashError SplashBitmap::writePNMFile(char *fileName) {
     }
     break;
 
- case splashModeRGB8Qt:
-    fprintf(f, "P6\n%d %d\n255\n", width, height);
-    row = data;
-    for (y = 0; y < height; ++y) {
-      p = row;
-      for (x = 0; x < width; ++x) {
-	fputc(splashRGB8R(p), f);
-	fputc(splashRGB8G(p), f);
-	fputc(splashRGB8B(p), f);
-	p += 4;
-      }
-      row += rowSize;
-    }
-    break;
-
-  case splashModeARGB8:
-    fprintf(f, "P6\n%d %d\n255\n", width, height);
-    row = data;
-    for (y = 0; y < height; ++y) {
-      p = row;
-      for (x = 0; x < width; ++x) {
-	fputc(splashARGB8R(p), f);
-	fputc(splashARGB8G(p), f);
-	fputc(splashARGB8B(p), f);
-	p += 4;
-      }
-      row += rowSize;
-    }
-    break;
-
-  case splashModeBGRA8:
-    fprintf(f, "P6\n%d %d\n255\n", width, height);
-    row = data;
-    for (y = 0; y < height; ++y) {
-      p = row;
-      for (x = 0; x < width; ++x) {
-	fputc(splashBGRA8R(p), f);
-	fputc(splashBGRA8G(p), f);
-	fputc(splashBGRA8B(p), f);
-	p += 4;
-      }
-      row += rowSize;
-    }
-    break;
-
 #if SPLASH_CMYK
   case splashModeCMYK8:
-  case splashModeACMYK8:
     // PNM doesn't support CMYK
     break;
 #endif
   }
-
-  fclose(f);
   return splashOk;
 }
 
+
 void SplashBitmap::getPixel(int x, int y, SplashColorPtr pixel) {
   SplashColorPtr p;
 
@@ -216,50 +197,43 @@ void SplashBitmap::getPixel(int x, int y, SplashColorPtr pixel) {
   switch (mode) {
   case splashModeMono1:
     p = &data[y * rowSize + (x >> 3)];
-    pixel[0] = (p[0] >> (7 - (x & 7))) & 1;
+    pixel[0] = (p[0] & (0x80 >> (x & 7))) ? 0xff : 0x00;
     break;
   case splashModeMono8:
     p = &data[y * rowSize + x];
     pixel[0] = p[0];
     break;
-  case splashModeAMono8:
-    p = &data[y * rowSize + 2 * x];
-    pixel[0] = p[0];
-    pixel[1] = p[1];
-    break;
   case splashModeRGB8:
-  case splashModeBGR8:
     p = &data[y * rowSize + 3 * x];
     pixel[0] = p[0];
     pixel[1] = p[1];
     pixel[2] = p[2];
     break;
-  case splashModeRGB8Qt:
+  case splashModeXBGR8:
     p = &data[y * rowSize + 4 * x];
     pixel[0] = p[2];
     pixel[1] = p[1];
     pixel[2] = p[0];
+    pixel[3] = p[3];
+    break;
+  case splashModeBGR8:
+    p = &data[y * rowSize + 3 * x];
+    pixel[0] = p[2];
+    pixel[1] = p[1];
+    pixel[2] = p[0];
     break;
-  case splashModeARGB8:
-  case splashModeBGRA8:
 #if SPLASH_CMYK
   case splashModeCMYK8:
-#endif
     p = &data[y * rowSize + 4 * x];
     pixel[0] = p[0];
     pixel[1] = p[1];
     pixel[2] = p[2];
     pixel[3] = p[3];
     break;
-#if SPLASH_CMYK
-  case splashModeACMYK8:
-    p = &data[y * rowSize + 5 * x];
-    pixel[0] = p[0];
-    pixel[1] = p[1];
-    pixel[2] = p[2];
-    pixel[3] = p[3];
-    pixel[4] = p[4];
-    break;
 #endif
   }
 }
+
+Guchar SplashBitmap::getAlpha(int x, int y) {
+  return alpha[y * width + x];
+}
